# django-kerberos - SPNEGO/Kerberos authentication for Django applications
# Copyright (C) 2014-2019 Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import warnings


from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend

from .utils import check_password


logger = logging.getLogger('django_kerberos')

UserModel = get_user_model()


class KerberosBackend(ModelBackend):
    def user_for_principal(self, request, principal):
        try:
            warnings.warn('example backend do not use in production!')
            return UserModel.objects.get(username=str(principal))
        except UserModel.DoesNotExist:
            logger.warning('no user found for principal %s', principal)
            return None

    def authenticate(self, request, principal=None):
        return self.user_for_principal(request, principal)


class KerberosPasswordBackend(KerberosBackend):
    def principal_for_user(self, user, username):
        return username

    def authenticate(self, request, username=None, password=None, **kwargs):
        '''Verify username and password using Kerberos'''
        if username is None:
            username = kwargs.get(UserModel.USERNAME_FIELD)
        if username is None or password is None:
            return

        try:
            user = UserModel.objects.get(username=username)
        except UserModel.DoesNotExist:
            return

        principal = self.principal_for_user(user, username)

        if check_password(principal, password):
            if not user.check_password(password):
                user.set_password(password)
                user.save()
            return user
        else:
            return None
