# django-kerberos - SPNEGO/Kerberos authentication for Django applications
# Copyright (C) 2014-2019 Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

import base64
import logging

import gssapi
import gssapi.exceptions

from django import http
from django.conf import settings
from django.contrib import messages
from django.utils.http import is_safe_url
from django.views.generic.base import View

from django.contrib.auth import authenticate, login as auth_login, REDIRECT_FIELD_NAME

logger = logging.getLogger('django_kerberos')


class NegotiateFailed(Exception):
    pass


class NegotiateView(View):
    redirect_field_name = REDIRECT_FIELD_NAME

    def get_success_url_allowed_hosts(self):
        return {self.request.get_host()}

    def get_redirect_url(self):
        """Return the user-originating redirect URL if it's safe."""
        redirect_to = self.request.POST.get(
            self.redirect_field_name,
            self.request.GET.get(self.redirect_field_name, '')
        )
        url_is_safe = is_safe_url(
            url=redirect_to,
            allowed_hosts=self.get_success_url_allowed_hosts(),
            require_https=self.request.is_secure(),
        )
        return redirect_to if url_is_safe else settings.LOGIN_REDIRECT_URL

    def service_name(self):
        # force a service name, ex. :
        # server_name = 'HTTP@%s' % self.request.get_host()
        # return gssapi.Name(server_name, name_type=gssapi.NameType.hostbased_service)
        # without one, any service name in keytab will do
        return None

    def _add_www_authenticate(self, response, token=None):
        encoded_token = base64.b64encode(token).decode('ascii') if token else None
        logger.debug('sending challenge WWW-Authenticate: Negotiate %s', encoded_token)
        response['WWW-Authenticate'] = 'Negotiate%s' % (
            ' ' + encoded_token if token else '')

    def challenge(self):
        '''Send negotiate challenge'''
        return http.HttpResponse(status=401)

    def success(self, request, *args, user=None, **kwargs):
        '''Do something with the user we found'''
        auth_login(self.request, user)
        return self.redirect(request, *args, **kwargs)

    def redirect(self, request, *args, **kwargs):
        return http.HttpResponseRedirect(self.get_redirect_url(), status=307)

    def negotiate(self, request, *args, **kwargs):
        '''Try to authenticate the user using SPNEGO and Kerberos'''

        gss_name = self.service_name()
        if gss_name:
            logger.debug('using service principal name %s', gss_name)

        out_token = None
        response = None
        try:
            server_creds = gssapi.Credentials(usage='accept', name=gss_name)
        except gssapi.exceptions.GSSError as e:
            return self.error(request, message='gssapi credentials failure %s' % e, *args, **kwargs)
        else:
            in_token = None
            if request.META.get('HTTP_AUTHORIZATION', '').startswith('Negotiate '):
                authstr = request.META['HTTP_AUTHORIZATION'][10:]
                try:
                    in_token = base64.b64decode(authstr)
                except ValueError:
                    pass

            if in_token is not None:
                server_ctx = gssapi.SecurityContext(creds=server_creds, usage='accept')
                try:
                    out_token = server_ctx.step(in_token)
                except gssapi.exceptions.GSSError as e:
                    return self.error(request, message='gssapi security context failure %s' % e, *args, **kwargs)
                else:
                    if server_ctx.complete:
                        principal = server_ctx.initiator_name
                        user = authenticate(principal=principal)
                        if user:
                            logger.debug('found user %s for principal %s', user, principal)
                            response = self.success(request, user=user)
        if response is None:
            response = self.challenge()
        self._add_www_authenticate(response, out_token)
        return response

    def error(self, request, message, *args, **kwargs):
        logger.warning(message)
        messages.error(request, message)
        return self.redirect(request, *args, **kwargs)

    def get(self, request, *args, **kwargs):
        return self.negotiate(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        return self.negotiate(request, *args, **kwargs)

login = NegotiateView.as_view()
