# django-kerberos - SPNEGO/Kerberos authentication for Django applications
# Copyright (C) 2014-2019 Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import base64
import logging
import os
import pytest

import gssapi
import k5test
import k5test._utils

from django.contrib.auth import get_user_model, authenticate

User = get_user_model()


@pytest.fixture
def k5env():
    k5realm = k5test.K5Realm(krb5_config={
        'libdefaults': {
            'dns_canonicalize_hostname': False,
            'rdns': False,
        }
    })
    old_environ = os.environ.copy()
    try:
        os.environ.update(k5realm.env)
        k5realm.http_princ = 'HTTP/testserver@%s' % k5realm.realm
        k5realm.addprinc(k5realm.http_princ)
        k5realm.extract_keytab(k5realm.http_princ, k5realm.keytab)
        yield k5realm
    finally:
        os.environ.clear()
        os.environ.update(old_environ)
        k5realm.stop()


def test_login(k5env, client, caplog, db, monkeypatch):
    caplog.set_level(logging.DEBUG)

    response = client.get('/login/')
    assert response.status_code == 401

    service_name = gssapi.Name('HTTP@testserver', gssapi.NameType.hostbased_service)
    service_name.canonicalize(gssapi.MechType.kerberos)

    # first attempt
    ctx = gssapi.SecurityContext(usage='initiate', name=service_name)
    token = ctx.step()
    response = client.get('/login/',
                          HTTP_AUTHORIZATION='Negotiate %s' % base64.b64encode(token).decode('ascii'))

    assert response.status_code == 401
    assert '_auth_user_id' not in client.session

    # create an user...
    User.objects.create(username=k5env.user_princ)

    # and retry.
    ctx = gssapi.SecurityContext(usage='initiate', name=service_name)
    token = ctx.step()
    response = client.get('/login/',
                          HTTP_AUTHORIZATION='Negotiate %s' % base64.b64encode(token).decode('ascii'))

    assert response.status_code == 307
    assert client.session['_auth_user_id']

    # break service name resolution
    monkeypatch.setattr('django_kerberos.views.NegotiateView.service_name',
                        lambda self: gssapi.Name('HTTP@whatever.invalid', gssapi.NameType.hostbased_service))
    ctx = gssapi.SecurityContext(usage='initiate', name=service_name)
    token = ctx.step()
    response = client.get('/login/',
                          HTTP_AUTHORIZATION='Negotiate %s' % base64.b64encode(token).decode('ascii'),
                          HTTP_X_REQUESTED_WITH='XMLHttpRequest')
    assert response.status_code == 307
    cookies = response.cookies.output()
    assert 'gssapi security context failure' in caplog.text or 'gssapi credentials failure' in caplog.text
    assert 'gssapi security context failure' in cookies or 'gssapi credentials failure' in cookies


def test_password(k5env, db):
    user = User.objects.create(username=k5env.user_princ)

    k5env.run(['kdestroy'])

    assert authenticate(username=k5env.user_princ, password='nogood') is None
    assert authenticate(username=k5env.user_princ, password=k5env.password('user')) == user
    assert not os.path.exists(k5env.ccache)
